# Generated by roxygen2: do not edit by hand
S3method(plot,hlaAttrBagClass)
S3method(plot,hlaAttrBagObj)
S3method(predict,hlaAttrBagClass)
S3method(print,hlaAttrBagClass)
S3method(print,hlaAttrBagObj)
S3method(summary,hlaAlleleClass)
S3method(summary,hlaAttrBagClass)
S3method(summary,hlaAttrBagObj)
S3method(summary,hlaSNPGenoClass)
S3method(summary,hlaSNPHaploClass)
export(.Last.lib)
export(.onAttach)
export(.onLoad)
export(hlaSNPID)
export(.get_model_path)
export(kirParallelAttrBagging)
export(kirPredict)
export(predict.hlaAttrBagClass)
exportPattern("^[^\\\\.]")
importFrom(graphics,abline)
importFrom(graphics,text)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,sd)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(utils,download.file)
useDynLib(PONG2)
importFrom("utils", "capture.output")
